/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************/

#include "crackMShift.h"

// Pre:  None.
// Post: Initialize variables.
// Runtime: O(1)
crackMShift::crackMShift(unsigned int inMPass, unsigned int inMFile){
    passphrase = "";
    maxP = inMPass;
    maxF = inMFile;

    // Define Accepted Letter Frequencies
    frequency = new float[26];
    frequency[0]  = 0.082; // A
    frequency[1]  = 0.015; // B
    frequency[2]  = 0.028; // C
    frequency[3]  = 0.043; // D
    frequency[4]  = 0.127; // E
    frequency[5]  = 0.022; // F
    frequency[6]  = 0.020; // G
    frequency[7]  = 0.061; // H
    frequency[8]  = 0.070; // I
    frequency[9]  = 0.002; // J
    frequency[10] = 0.008; // K
    frequency[11] = 0.040; // L
    frequency[12] = 0.024; // M
    frequency[13] = 0.067; // N
    frequency[14] = 0.075; // O
    frequency[15] = 0.019; // P
    frequency[16] = 0.001; // Q
    frequency[17] = 0.060; // R
    frequency[18] = 0.063; // S
    frequency[19] = 0.091; // T
    frequency[20] = 0.028; // U
    frequency[21] = 0.010; // V
    frequency[22] = 0.023; // W
    frequency[23] = 0.001; // X
    frequency[24] = 0.020; // Y
    frequency[25] = 0.001; // Z
}

// Pre: All necessary variables are legal. 
// Post: Invalid valuables are handled. The specified file is opened, and
//       analyzed assuming a passphrase of length inLen, and passphrase is
//       set to the one with the best fit.
// Runtime: O(N + inLen) where N is the number of characters in the input file.
bool crackMShift::crack(unsigned int inLen, string inFileName){
    // Validate input
    if(inLen < 1 || inLen > maxP || inFileName.length() > maxF)
	return false; // Fail

    // Initialize and declare
    char tempC;
    unsigned int i, j, k, shift;
    float fit, curFit, tempF;
    passphrase = "";
    int** sampleFreqI = new int*[inLen];
    float** sampleFreqF = new float*[inLen];
    for(i = 0; i < inLen; i++){
	sampleFreqI[i] = new int[27];
	sampleFreqF[i] = new float[26];
	for(j = 0; j < 27; j++){
	    sampleFreqI[i][j] = 0;
	    if(j < 26)
		sampleFreqF[i][j] = 0.0;
	}	
    }
 
    // Verify input file validity.
    ifstream inFile(inFileName.c_str());
    if(!inFile.is_open())
	return false;
 
    // Validate input and determine frequencies.
    i = 0;
    while(inFile >> tempC){
	if(isspace(tempC))
	    ; // Do Nothing.
	if(!isalpha(tempC)){
	    inFile.close();
	    for(i= 0; i < inLen; i++)
		delete sampleFreqI[i];
	    delete sampleFreqI[i];
	    return false; // Fail
	}
	else{
	    sampleFreqI[i][int(toupper(tempC)) - int('A')]++;
	    sampleFreqI[i][26]++;
	    i++;
	    i = i % inLen;
	}
    }

    // Done w/ input file.
    inFile.close();

    // Get percentage frequencies.
    for(i = 0; i < inLen; i++){
	for(j = 0; j < 26; j++){
	    sampleFreqF[i][j] = float(sampleFreqI[i][j]) / float(sampleFreqI[i][26]);
	}
    }
    
    // Find Best Fit
    for(i = 0; i < inLen; i++){
	for(j = 0; j < 26; j++){
	    curFit = 0.0;
	    for(k = 0; k < 26; k++){
		
		// Use Error Sum of Squares Method.
		tempF = frequency[k] - sampleFreqF[i][(j + k) % 26];
		curFit += tempF * tempF;
	    }
	    if(j == 0){
		shift = 0;
		fit = curFit;
	    }else if(curFit < fit){
	    fit = curFit;
	    shift = j;
	    }
	}
	passphrase = passphrase + char(int('A') + shift);
    }

    // Clean up and return.
    for(i= 0; i < inLen; i++)
	delete sampleFreqI[i];
    delete sampleFreqI;
    return true; // Succeed
}

// Pre: None.
// Post: Passphrase is returned if valid, else NULL is returned.
// Runtime: O(1)
string crackMShift::getPass(){
    return passphrase;
}

// Pre:  None.
// Post: Deallocate used memory.
// Runtime: O(1)
crackMShift::~crackMShift(){
    if(frequency != NULL)
	delete frequency;
}
