/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************

 Description: This class divides the ciphertext into inLen groups, and uses
              frequency analysis to determine what shift was used to encipher 
              the group.

 ***********************************************************************************/

#ifndef __CRACKMSHIFT_H
#define __CRACKMSHIFT_H
using namespace std;
#include <string>
#include <fstream>

class crackMShift{

public:
    // Pre:  None.
    // Post: Initialize variables.
    // Runtime: O(1)
    crackMShift(unsigned int inMPass, unsigned int inMFile);

    // Pre: All necessary variables are legal. 
    // Post: Invalid valuables are handled. The specified file is opened, and
    //       analyzed assuming a passphrase of length inLen, and passphrase is
    //       set to the one with the best fit.
    // Runtime: O(N + inLen) where N is the number of characters in the input file.
    bool crack(unsigned int inLen, string inFileName);

    // Pre: None.
    // Post: Passphrase is returned if valid, else -1 is returned.
    // Runtime: O(1)
    string getPass();

    // Pre:  None.
    // Post: Deallocate used memory.
    // Runtime: O(1)
    ~crackMShift();
     
private:
    unsigned int maxP;
    unsigned int maxF;
    string passphrase;
    float* frequency;
};

#endif
