/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************/

#include "dictLookup.h"

// Pre:  Input in dictionary file is in
//       lexpgraphical order.
// Post: Construct dictionary in memory and set variables.
// Runtime: O(D) where D is the number of entries in the dictionary file.
dictLookup::dictLookup(void){
    // Validate and open file
    ifstream inFile("dictionary");
    if(!inFile.is_open()){
	printf("Cannot find file `dictionary'.\n");
	exit(-1);
    }

    // Process
    string tempW;
    unsigned int i;
    while(inFile >> tempW){
	for(i = 0; i < tempW.size(); i++)
	    if(!isalpha(tempW[i]))
		break; // Not a fatal error. Proceed.
	if(i == tempW.size())
	    theList.insert(tempW);//PROCESS VALID DICT WORDS
    }
    inFile.close();
}

// Pre:  inWord is a valid string.
// Post: if word is in dictionary, return true. Else, false.
// Runtime: O(D) where D is the number of entries in the dictionary file
//          for the first time it is run, and then O(log(D)) thereafter.
bool dictLookup::lookup(string inWord){
    return theList.search(inWord);
}

// Pre:  inWord is a valid string.
// Post: if word is in dictionary, return true. Else, false.
// Runtime: O(D) where D is the number of entries in the dictionary file
//          for the first time it is run, and then O(log(D)) thereafter.
bool dictLookup::search(string inWord){
    return theList.search(inWord);
}

// Pre:  inFile is a valid string
// Post: Attempts to open inFile. If it fails, returns -1.0, else returns
//       the % of words in inFile that are in English. Returns -2.0 if
//       inFile does not have spaces, and thus no words.
// Runtime: O(N*log(D))
float dictLookup::percentEnglish(string inFileName){
//    ifstream inFile(inFileName.c_str());
    FILE* in = fopen(inFileName.c_str(), "r");
    if(in == NULL)
	return -1.0; // Invalid filename.
    
    unsigned long int numEnglish(0), numWords(0);
    char inChar;
    string inWord;
    while(EOF != fscanf(in, "%c", &inChar)){
	if(isspace(inChar)){
	    if(numWords == ULONG_MAX - 1)
		break;
	    if(theList.search(inWord))
		numEnglish++;
	    numWords++;
	    inWord = "";
	}else if(isalpha(inChar))
	    inWord = inWord + char(toupper(inChar));
    }
    fclose(in);
    if(numWords < 5) // No spaces means < 5 words.
	return -2.0;

    return 100.0 * float(numEnglish)/float(numWords);
}

// Pre:  theList is a valid stringList
// Post: return theList.size()
// Runtime: O(1)
unsigned int dictLookup::size(){
    return theList.size();
}
