/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************

 Description: This class looks up words in the dictionary. 

 ***********************************************************************************/

#ifndef __DICTLOOKUP_H
#define __DICTLOOKUP_H

using namespace std;

#include <fstream>
#include "stringList.h"

class dictLookup{

public:

    // Pre:  Input in dictionary file is in
    //       lexpgraphical order.
    // Post: Construct dictionary in memory and set variables.
    // Runtime: O(D) where D is the number of entries in the dictionary file.
    dictLookup();

    // Pre:  inWord is a valid string.
    // Post: if word is in dictionary, return true. Else, false.
    // Runtime: O(D) where D is the number of entries in the dictionary file
    //          for the first time it is run, and then O(log(D)) thereafter.
    bool lookup(string inWord);

    // Pre:  inWord is a valid string.
    // Post: if word is in dictionary, return true. Else, false.
    // Runtime: O(D) where D is the number of entries in the dictionary file
    //          for the first time it is run, and then O(log(D)) thereafter.
    bool search(string inWord);

    // Pre:  inFile is a valid string
    // Post: Attempts to open inFile. If it fails, returns -1.0, else returns
    //       the % of words in inFile that are in English. Returns -2.0 if
    //       inFile does not have spaces, and thus no words.
    // Runtime: O(N*log(D))
    float percentEnglish(string inFileName);

    // Pre:  theList is a valid stringList
    // Post: return theList.size()
    // Runtime: O(1)
    unsigned int size();

private:
    stringList theList;
};

#endif
