/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************

 Description: This class performs the various functionalities of the friedman test.

 ***********************************************************************************/


// Note that is is using n = 26 characters in the alphabet. If it needs to be
// used otherwise, it could be easily modified.

#ifndef __FRIEDMAN_H
#define __FRIEDMAN_H

using namespace std;
#include <fstream>
#include <string>

class friedman{

public:
    // Pre:  None.
    // Post: All internal variables are initialized
    // Runtime: O(1);
    friedman(unsigned int inMaxFile);

    // Pre:  Internal vars have been initialized properly and inFileName is a
    //       valid string.
    // Post: Calculate the approximate keylength, and set estKeyLen equal to
    //       it. If successful, return true, else return false.
    // Runtime: O(N) where N is the length of the input file.
    bool calcKeyLen(string inFileName);

    // Pre:  estKeyLen is a valid float.
    // Post: Return estKeyLen
    // Runtime: O(1)
    float getEstKeyLen();

    // Pre:  prevIC is a valid float.
    // Post: Return prevIC
    // Runtime: O(1)
    float getPrevIC();

    // Pre:  Input variables are valid.
    // Post: returns the average IC for the file obtained by dividing it into inLen
    //       groups.
    // Runtime: O(N + inLen)
    float getIC(string inFileName, unsigned int inLen);

    // Pre:  None.
    // Post: return EnglishIC
    // Runtime: O(1)
    float getEnglishIC();

    // Pre:  inIC is a valid number 0 <= inIC <= 1
    // Post: If inIC is valid, return the absolute value of the difference between
    //       it and EnglishIC, else return -1.0.
    // Runtime: O(1)    
    float diffEnglishIC(float inIC);

private:
    
    unsigned int maxFileLen;
    float estKeyLen;
    float EnglishIC;
    float prevIC;
		     
};

#endif
