
// Eric Reed
// 2-01-05
// Kasiski Test - attempts to guess length of keyword
//
// Used by Matthew Berntsen for an Honors Thesis with permission.

/*
NOTES: (any of these can be modified below to suit a specific need)
 ~ only the first 5000 characters will be analyzed if string is longer
 ~ will automatically remove non-alphabetic characters for analysis
 ~ converts all characters to same case (upper) for analysis
 ~ starts looking for patterns of length 4 and checks until none of a certain
    length are found
 ~ requires the string being analyzed to be at least 50 characters long

RUNTIME: O(N^3)
*/


#include <iostream>
#include <string>
#include <vector>

using namespace std;

int main(int argc, char* argv[]){ // O(N^3)
  string input, temp("");
  bool anyTest = false;
  unsigned int IN_MAX = 50000;
  unsigned int i, j;
  // will only examine first 5000 characters if string happens to be longer
  while(getline(cin, input) && temp.length() < IN_MAX){ // O(IN_MAX) = O(1)
    if("" == input)
      break;

    // removes any non-letter occurrences
    for(i = 0; i < input.length(); i++){
      if((65 <= input[i] && 122 >= input[i]) && (!((90 < input[i]) && (97 > input[i]))))
	temp += input[i];
	if(temp.length() >= IN_MAX)
	  break;
    }
    
    // converts to all uppercase
    for(i = 0; i < temp.length(); i++){
      if(islower(temp[i]))
	temp[i] = toupper(temp[i]);
    }
  }
  cout << endl;
  cout << "Analyzing " << temp.length() << " characters...please be patient...\n";
  cout << endl;

  // ASSERT: TEMP HOLDS THE ENTIRE CIPHERTEXT IN ONE CONTINUOUS STREAM OF CHARS

  // checks for patterns of length z, starting at length 4 until no more found
  bool anyMore = true;
  for(int z = 4; anyMore; z++){ // O(N^3)
    vector <string> dup_str(0);
    vector <int> dup_cnt(0);

    for(i = 0; i <= temp.length() - z; i++){ // for 1 O(N^2)
      string checkz = temp.substr(i, z);
      int present = 0;
      for(unsigned int q = 0; q < dup_str.size(); q++){ // for 4 O(N)
	if(checkz == dup_str[q])
	  present++;
      } // for 4

      if(present == 0){ // if 5
	int count = 0;
      
	for(j = 0; j <= temp.length() - z; j++){ // for 2 O(N)
	  if(checkz == temp.substr(j, z))
	    count++;
	} // for 2
      
	//if(count > 1){ // if 3
	if(count > 2){ // if 3
	  dup_str.push_back(checkz);
	  dup_cnt.push_back(count);
	} // if 3
      } // if 5
    } // for 1
  
    if(dup_str.size() > 0){
      anyTest = true;
       cout << "Found " << dup_str.size() << " unique groups of " << z << " letters.\n";
    }
    else{
      cout << "no groups of " << z << " located.\n";
      anyMore = false;
    }
  } // MASTER

  if(!anyTest)
    //cout << "No patterns were located, or your text is < 50 chars long:"
    cout << "No patterns were located: "
	 << " length = " << temp.length() << endl;

  return 0;
}
