// Use or parent pointers (doubly linked) considerably reduces runtime for long strings.
// Modified splay tree.
#include "stringBST.h"
#include <iostream>
stringBST::stringBST(bool inLoc){
    loc = inLoc;
    root = NULL;
    size = 0;
}

stringBST::~stringBST(){
    printf("DECONSTRUCT\n");
    del(root);
}

bool stringBST::insert(string inS){
    return insert(inS, 0);
}


bool stringBST::insert(string inS, int inLoc){
    for(int i = 0; i < inS.size(); i++){
	if(!isalpha(inS[i]))
	    return false;
	inS[i] = toupper(inS[i]);
    }
    SBSTNode* cur;
    bool cont = true;
    SBSTNode* tempN = findp(inS);

    // Handle duplicate entries.
    if(tempN != NULL){
	if(tempN->left != NULL && tempN->left->data == inS)
	    cur = tempN->left;
	else if(tempN->right !=NULL && tempN->right->data == inS)
	    cur = tempN->right;
	else
	    cont = false; // Not a duplicate. Don't process as such.

	if(cont){
	    splay(cur);
	    // Handle duplicate.
	    if(loc){
		cur->locations->push_back(inLoc);
		return true;
	    }else
		return false;
	    return true;
	}
    }else if(root != NULL && root->data == inS){ //Found in root
	if(loc){
	    root->locations->push_back(inLoc);
	    return true;
	}else
	    return false;
	return true;
    }

    // Handle unique insertions.
    cur = new SBSTNode;
    cur->data = inS;
    if(root == NULL){ // Empty Tree.
	root = cur;
    }
    else{ // Non-empty tree unique insertion. Splaydom!!!
	// NOTE: This assumes that findp works correctly!!
	if(lessThan(inS, tempN->data)){
	    tempN->left = cur;
	}else{
	    tempN->right = cur;
	}
    }

    // Set values of cur (inserted node) and return.
    if(loc){
	cur->locations = new vector<unsigned int>;
	cur->locations->push_back(inLoc);
    }else
	cur->locations = NULL;
    size++;
    cur->left = cur->right = NULL;
    cur->parent = tempN;
//    cout << "Insert: " << cur->data;
//    if(cur->parent != NULL)
//	cout << " w/ parent: " << cur->parent->data;
//    cout << endl;
    splay(cur);
//    print(true);
//    cout << "Root: "<< root->data << endl;
//    cout << "cur: " << cur-> data << " " << cur->parent << endl;
    return true;
}

bool stringBST::search(string inS){
    for(int i = 0; i < inS.size(); i++){
	if(!isalpha(inS[i]))
	    return false;
	inS[i] = toupper(inS[i]);
    }
    SBSTNode* tempN = find(inS);
    if(tempN == NULL)
	return false;
    splay(tempN);
    return true;
}

bool stringBST::remove(string inS){
    for(int i = 0; i < inS.size(); i++)
	inS[i] = toupper(inS[i]);
/*
    SBSTNode* tempNP = findp(inS);
    SBSTNode* tempN = NULL;
    bool lt = lessThan(inS, tempNP->data);
    if(lt)
	tempN = tempNP->left;
    else
	tempN = tempNP->right;

    // No Children, delete.
    if(tempN->left == NULL && tempN->right == NULL) // Delete Node.
	del(tempN);

    // Right child only, promote right, delete.
    else if(tempN->left == NULL){
	if(lt)
	    tempNP->left = tempN->right;
	else
	    tempNP->right = tempN->right;
	tempN->right = NULL;
	if(tempN->locations != NULL)
	    delete tempN -> locations;
	delete tempN;
    }

    // Left child only, promote left, delete.
    else if(tempN->right == NULL){
	if(lt)
	    tempNP->left = tempN->left;
	else
	    tempNP->right = tempN->left;
	tempN->left = NULL;
	if(tempN->locations != NULL)
	    delete tempN -> locations;
	delete tempN;
    }

    else //Two children. Start working down the tree.*/
}
    
// Pre:  The tree is properly constructed.
// Post: Traverses the tree to find the value inS. If successful, returns its
//       location. Else, returns NULL.
// Runtime: O(log(N)) where N is the number of nodes currently in the tree.
SBSTNode* stringBST::find(string inS){
    SBSTNode* tempN = findp(inS);
    if(tempN != NULL){
	if(lessThan(inS, tempN->data))
	    tempN = tempN->left;
	else
	    tempN = tempN->right;
    }
    return tempN;
}

// Pre:  The tree is properly constructed.
// Post: Traverses the tree to find the node who is or would be the parent of
//       the node containing inS.
// Runtime: O(log(N)) where N is the number of nodes currently in the tree.
SBSTNode* stringBST::findp(string inS){
    SBSTNode* tempN = root;
    if(root != NULL && root->data == inS)
	return NULL;
    while(tempN != NULL){
	if(lessThan(inS, tempN->data)){
	    if(tempN->left == NULL || tempN->left->data == inS)
		break;
	    tempN = tempN->left;
	}else{
	    if(tempN->right == NULL || tempN->right->data == inS)
		break;
	    tempN = tempN -> right;
	}
    }
    return tempN;
}

// Pre:  inSL and inSR are valid strings.
// Post: If inSL comes lexographically before inSR, return true. Else, return
//       false.
// Runtime: O(inSL.size() + inSR.size())
bool stringBST::lessThan(string inSL, string inSR){
    for(int i = 0; i < inSL.size() && i < inSR.size(); i++){
	if(inSL[i] < inSR[i])
	    return true;
	if(inSL[i] > inSR[i])
	    return false;
    }
    // At this point, same up to termination.
    if(inSL.size() < inSR.size())
	return true;
    return false;
}

// Pre:  The tree is properly constructed.
// Post: Deallocate all used memory and destroys the tree.
// Runtime: O(N) where N is the number of nodes in the tree.
void stringBST::del(SBSTNode* inNode){
    if(inNode != NULL){
	del(inNode -> left);
	del(inNode -> right);
	if(inNode->locations != NULL)
	    delete inNode -> locations;
	delete inNode;
	inNode = NULL;
    }
}

// Pre:  The tree is properly constructed.
// Post: Do an inorder print of the tree's contents. If inNL, seperate words
//       with newlines, otherwise seperate with spaces.
// Runtime: O(N) where N is the size of the tree.
void stringBST::print(bool inNL){
    prHelper(root, inNL);
    printf("\n");
}

// Pre:  The tree is properly constructed.
// Post: Do an inorder print of the subtree with inN as its root. If inNL,
//       seperate words with newlines, otherwise seperate with spaces.
// Runtime: O(N) where N is the size of the tree.
void stringBST::prHelper(SBSTNode* inN, bool inNL){
/* */
   if(inN != NULL){
	prHelper(inN->left, inNL);
	if(inN->data.size() > 0)
//	    printf(inN->data.c_str());
	if(inNL)
	    printf("\n");
	else
	    printf(" ");
	prHelper(inN->right, inNL);
    }
/**//**
    if(inN != NULL){
	cout << inN << " ";
	printf(inN->data.c_str());
	cout << " " << inN->parent << " " << inN->left << " " << inN->right;
	if(inNL)
	    printf("\n");
	else
	    printf(" ");
	prHelper(inN->left, inNL);
	prHelper(inN->right, inNL);
    }/**/
    
}

// Pre: The tree is properly constructed, inN and inN->left are not NULL
// Post: Perform a right rotate on inN and inN->left
// Runtime: O(1)
void stringBST::rightRotate(SBSTNode* inN){
/*    if(inN != NULL && inN->left != NULL){
	SBSTNode* newPar = inN->left;
	if(inN == root)
	    root = newPar;
	else{
	    if(inN->parent->left == inN)
		inN->parent->left = newPar;
	    else if(inN->parent->right == inN)
		inN->parent->right = newPar;
	    else{
		printf("Error in stringBST.cc tree structure.\n");
		exit(-1);
	    }
	}
	
	newPar->parent = inN->parent;
	inN->parent = inN;
	inN->right = newPar->left;
	newPar->left = inN;
    }

/**/
//    cout << "right " << inN->data << endl;
    if(inN != NULL && inN->left != NULL){
	SBSTNode* tempN = inN->left;
	inN->left = tempN->right;
	if(tempN->right != NULL)
	    tempN->right->parent = inN;
	tempN->right = inN;
	if(inN == root){
	    root = tempN;
//	    cout << "ROOTAGE\n";
	}
	tempN->parent = inN->parent;
	inN->parent = tempN;
	if(tempN->parent != NULL){
	    if(inN == tempN->parent->left)
		tempN->parent->left = tempN;
	    else if(inN == tempN->parent->right)
		tempN->parent->right = tempN;
	    else{
		printf("Error in stringBST.cc tree structure.\n");
		exit(-1);
	    }
	}
    }/**/
}

// Pre: The tree is properly constructed, inN and inN->right are not NULL
// Post: Perform a left rotate on inN and inN->right
// Runtime: O(1)
void stringBST::leftRotate(SBSTNode* inN){
/*    if(inN != NULL && inN->right != NULL){
	SBSTNode* newPar = inN->right;
	if(inN == root)
	    root = newPar;
	else if(inN->parent->left == inN)
	    inN->parent->left = newPar;
	else if(inN->parent->right == inN)
	    inN->parent->right = newPar;
	else{
	    printf("Error in stringBST.cc tree structure.\n");
	    exit(-1);
	}
	
	inN->right = newPar->left;
	newPar->left = inN;
    }

/**/
//    cout << "left " << inN->data<< endl;
    if(inN != NULL && inN->right != NULL){
	SBSTNode* tempN = inN->right;
	inN->right = tempN->left;
	if(tempN->left != NULL)
	    tempN->left->parent = inN;
	tempN->left = inN;
	if(inN == root){
	    root = tempN;
//	    cout << "ROOTAGE\n";
	}
	tempN->parent = inN->parent;
	inN->parent = tempN;
	if(tempN->parent != NULL){
	    if(inN == tempN->parent->left)
		tempN->parent->left = tempN;
	    else if(inN == tempN->parent->right)
		tempN->parent->right = tempN;
	    else{
		printf("Error in stringBST.cc tree structure.\n");
		exit(-1);
	    }
	}
    }
}


void stringBST::splay(SBSTNode* inN){
    // Not actual splay insertion. Root insertion.

    
//    cout << "Splay: " << inN->data << endl;
    while(inN != root){
    //   if(inN != root){
/*
	cout << "Root: "<< root->data << endl;
	cout.flush();
	print(true);
	if(inN->parent == NULL || 
	   (inN->parent->left == NULL && inN->parent->right == NULL)){
	    printf("Error in stringBST.cc tree structure.\n");
	    exit(-1);
	}
	if(inN->parent->left != NULL && inN == inN->parent->left)
	    rightRotate(inN->parent);
	else if(inN->parent->right != NULL && inN == inN->parent->right)
	    leftRotate(inN->parent);
	else{
	    printf("Error in stringBST.cc tree structure.\n");
	    exit(-1);
	    }/**/
	
	if(inN == inN->parent->left){
	    if(inN->parent == root)
		rightRotate(root);
	    else if(inN->parent == inN->parent->parent->left){
		// Zig-Zig
		rightRotate(inN->parent->parent);
		rightRotate(inN->parent);
	    }else if(inN->parent == inN->parent->parent->right){
		// Zig-Zag
		rightRotate(inN->parent);
		leftRotate(inN->parent);
		
	    }else{
		printf("Problem in stringBST tree structure.\n");
		exit(-1);
	    }
	}else if(inN == inN->parent->right){
	    if(inN->parent == root)
		leftRotate(root);
	    else if(inN->parent == inN->parent->parent->right){
		// Zig-Zig
		leftRotate(inN->parent->parent);
		leftRotate(inN->parent);
	    }else if(inN->parent == inN->parent->parent->left){
		// Zig-Zag
		leftRotate(inN->parent);
		rightRotate(inN->parent);
	    }else{
		printf("Problem in stringBST tree structure.\n");
		exit(-1);
	    }
	    
	}else{
	    printf("Problem in stringBST tree structure.\n");
	    exit(-1);
	    }/**/
    }/**/
}
