/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************

 Description: This class allows for fast creation [O(N)] and searches
              [O(log(N))] of a series of consecutively input strings.

 ***********************************************************************************/

#ifndef __STRINGLIST_H
#define __STRINGLIST_H

using namespace std;
#include <string>

struct strListNode{
    string data;
    strListNode* next;
};

class stringList{

public:

    // Pre:  None.
    // Post: Set initial values.
    // Runtime: O(1)
    stringList();

    // Pre:  List contents are contiguous and valid. if(loc), for all list elements
    //       pos != NULL.
    // Post: All allocated memory is deallocated.
    // Runtime: O(numElements)
    ~stringList();

    // Pre:  numElements is a valid unsigned int.
    // Post: return numElements
    // Runtime: O(1)
    unsigned int size();

    // Pre:  input is a valid string.
    // Post: inS is inserted into the list.
    // Runtime: O(1) as it is a linked list insertion.
    bool insert(string inS);

    // Pre:  inS is a valid string and the list is legally constructed.
    // Post: searches the list for inS. Returns true if it finds it, else false.
    // Runtime: O(log(numElements))
    bool search(string inS);

    // Pre:  The list is configured properly.
    // Post: Prints the list string contents.
    // Runtime: O(numElements)
    void print();

private:

    // Pre:  All internal data is valid.
    // Post: The linked list of data is transfered into an array for searching. 
    //       prepDone = true and start & end are deleted and set to NULL.
    // Runtime: O(numElements)
    void prep();

    // Pre:  inS is a valid string and the list is legally constructed.
    // Post: searches the list for inS. Returns the index if it finds it, else 0xffffffff
    // Runtime: O(N) the first time it is run (runs prep()), and O(Log(N)) thereafter.
    unsigned int find(string inS);

    /** Private Variables **/
    unsigned int numElements;
    string* dataStr;
    strListNode* start;
    strListNode* end;
    bool prepDone;
};


#endif
