/***********************************************************************************
 *                                                                                 *
 * Matthew C. Berntsen, Bucknell University 2005                                   *
 * Thesis Project for Honors in Computer Science                                   *
 * Spring 2005                                                                     *
 *                                                                                 *
 * http://mattberntsen.net?page=thesis                                             *
 *                                                                                 *
 ***********************************************************************************/

#include "vigCrypt.h"

// Pre:  inMaxP and inMaxF are valid.
// Post: maxP and maxF are set
// Runtime O(1)
vigCrypt::vigCrypt(unsigned int inMaxF, unsigned int inMaxP){
    if(inMaxF < 10)
	maxF = 10;
    else
	maxF = inMaxF;
    if(inMaxP < 10)
	maxP = 10;
    else
	maxP = inMaxP;
}

// Pre: passphrase, encrypt, file and mode are all valid.
// Post: values are initialized accordingly.
// Runtime: O(M + P) where M is the number of characters in the input file and
//          P is the length of the passphrase
bool vigCrypt::crypt(int mode, bool encrypt, string inPass, string inFileName){

    counter = 0;
    string ext, tempS;
    tableau = NULL;
    passI = NULL;
    FILE* in = NULL;
    FILE* out = NULL;
    if(encrypt)
	ext = ".enc";
    else
	ext = ".dec";
    
    // Validate Passphrase & Mode & do Prep Work.
    if(inPass.length() == 0 || inPass.length() > maxP || mode < 1 || mode > 2)
	return false;
    for(i = 0; i < inPass.length(); i++)
	if(!isalpha(inPass[i]))
	    return false;
    if(inFileName.length() > maxF)
	return false;
    in = fopen(inFileName.c_str(), "r");
    out = fopen(string(inFileName + ext).c_str(), "w");
    if(out == NULL || in == NULL)
	return false;
    passI = new short int[inPass.length()];

    for(i = 0; i < inPass.length(); i++){
	if(mode == 2 && !encrypt)
	    passI[i] = (26 - (toupper(inPass[i]) - 'A')) % 26; 
	else
	    passI[i] = toupper(inPass[i]) - 'A';
    }

    // Print Info
    if(encrypt)
	printf("\tEncrypting `");
    else
	printf("\tDecrypting `");
    printf(inFileName.c_str());

    // Process Encoding / Decoding
    switch(mode){
    case 1: // Use tableau
	printf("' %s", "Using Tableau Method");

	// Initialize Tableau
	tableau = new char*[26];
	for (i = 0; i < 26; i++){
	    tableau[i] = new char[26];
	    for(j = 0; j < 26; j++){
		tableau[i][j] = char((i + j) % 26 + int('A'));
	    }
	}
	
	// Process
	while(EOF != fscanf(in, "%c", &tempC)/*inFile >> tempC*/){
	    if(isalpha(tempC)){
		tempC = toupper(tempC);
		// Encrypt
		if(encrypt)
		    fprintf(out, "%c", tableau[int(tempC - 'A')][passI[counter]]);
		
		// Decrypt
		else{
		    i = 0;
		    while(i < 25 && tableau[i][passI[counter]] != tempC)
			i++;
		    tempC = char(i + int('A'));
		    fprintf(out, "%c", tempC);
		}
		counter++;
		counter = counter % inPass.length();
	    }else
		fprintf(out, "%c", tempC);	
	}
	break;
	
    case 2: // Use 'smart' method
	printf("' %s", "Using Computational Method");

	while( EOF != fscanf(in, "%c", &tempC)/*inFile >> tempC*/){
	    if(isalpha(tempC)){
		tempC = toupper(tempC);
		fprintf(out, "%c",  char((int(tempC) - int('A') + passI[counter]) % 26 + 
				    int('A')));
		counter++;
		counter = counter % inPass.length();
	    }else
		fprintf(out, "%c", tempC);
	}
    break;
    default:
	return false;
    }

    // Final Print
    printf(", Passphrase: %s\n", inPass.c_str());

    // Clean up, and return
    fclose(in);
    fclose(out);
    return true;
}

// Pre: None.
// Post: Any allocated memory is deallocated.
// Runtime O(1)
vigCrypt::~vigCrypt(){
    if(passI != NULL)
	delete passI;
    if (tableau != NULL){
	for(int i = 0; i < 26; i++)
	    delete tableau[i];
	delete tableau;
    }
}
